//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2015 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <stdio.h>
#include <cpctelera.h>
#include "jugador.h"
#include "bulletp.h"
#include "mapa.h"
#include "entities.h"
#include "enemy.h"
#include <string.h>

#define LIMITE_DERECHO ANCHO_PANTALLA - G_JUGADOR_00_W
#define LIMITE_ABAJO LARGO_PANTALLA - G_JUGADOR_00_H

u8 cont = 0;
TUnidad tank;
TBullet bala;

void drawBullet(){
  u8* pvmen = cpct_getScreenPtr(CPCT_VMEM_START, bala.x, bala.y);
  cpct_drawSpriteMaskedAlignedTable (bala.sprite, pvmen, G_BULLETP_0_W, G_BULLETP_0_H, g_tablatrans);
}

void deleteBullet(){
  cpct_etm_drawTileBox2x4 (bala.px/2, (bala.py-ORIGEN_MAPA_Y)/4, 3, 4, 40, ORIGEN_MAPA, getMapa());
}

void initBullet(){
  bala.disparada = 0;
  deleteBullet();
  bala.x = tank.x;
  bala.y = tank.y;
}

void reDrawBullet(){
  deleteBullet();
  bala.px=bala.x;
  bala.py=bala.y;
  drawBullet();
}

void deleteTank(){
  cpct_etm_drawTileBox2x4 (tank.px/2, (tank.py-ORIGEN_MAPA_Y)/4, 4, 5, 40, ORIGEN_MAPA, getMapa());
}

void drawTank(){
  u8* pvmen = cpct_getScreenPtr(CPCT_VMEM_START, tank.x, tank.y);
  cpct_drawSpriteMaskedAlignedTable (tank.sprite, pvmen, G_JUGADOR_00_W, G_JUGADOR_00_H, g_tablatrans);
}

void reDrawTank(){
  deleteTank();
  tank.px=tank.x;
  tank.py=tank.y;
  drawTank();
}

void startGame(){
  tank.x = tank.px = bala.x = bala.px = 4;
  tank.y = tank.py = bala.y = bala.py = 160;
  tank.mover = NO;
  tank.sprite = g_jugador_00;
  tank.vivo = 0;
  tank.tipo = 2;
  drawTank();
}

void moverIzq(){
  if(tank.x > 0 && colision(0,tank.x,tank.y,G_JUGADOR_00_W,G_JUGADOR_00_H) == 0){
    --tank.x;
    tank.mover = SI;
    tank.sprite = g_jugador_06;
    if(bala.disparada == 0){
      bala.x = tank.x;
      bala.y = tank.y;
    }
  }
}

void shootLeft(){
  u8 i = 0;
  for(i = 0; i < 3 && getVivoE(i) == 0; i++){
    if(comprobarMuerte(bala.x,bala.y,getXenemy(i),getYenemy(i),3,6) == 1){
      if(colision(0,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H) == 0){
        if(bala.disparada == 0){
          bala.x = tank.x - 6;
          bala.sprite = g_bulletp_3;
          bala.disparada = 1;
        }
        else{
          bala.x = bala.x - 2;
          reDrawBullet();
        }
      }
      else{
        modificarMapa(0,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H);
        bala.x = tank.x;
        bala.y = tank.y;
        deleteBullet();
        bala.disparada = 0;
        break;
      }
    }
    else{
      matarE(i);
      bala.x = tank.x;
      bala.y = tank.y;
      deleteBullet();
      bala.disparada = 0;
      break;
    }
  }
  if(bala.x <= 0){
    initBullet();
  }
}

void moverDerch(){
  if(tank.x < LIMITE_DERECHO && colision(2,tank.x,tank.y,G_JUGADOR_00_W,G_JUGADOR_00_H) == 0){
    ++tank.x;
    tank.mover = SI;
    tank.sprite = g_jugador_00;
    if(bala.disparada == 0){
      bala.x = tank.x;
      bala.y = tank.y;
    }
  }
}

void shootRight(){
  u8 i = 0;
  for(i = 0; i < 3 && getVivoE(i) == 0; i++){
    if(comprobarMuerte(bala.x,bala.y,getXenemy(i),getYenemy(i),3,6) == 1){
      if(colision(2,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H) == 0){
        if(bala.disparada == 0){
          bala.x = tank.x + G_JUGADOR_00_W;
          bala.sprite = g_bulletp_1;
          bala.disparada = 1;
        }
        else{
          bala.x = bala.x+2;
          reDrawBullet();
        }
      }
      else{
        modificarMapa(2,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H);
        bala.x = tank.x;
        bala.y = tank.y;
        deleteBullet();
        bala.disparada = 0;
        break;
      }
    }
    else{
      matarE(i);
      bala.x = tank.x;
      bala.y = tank.y;
      deleteBullet();
      bala.disparada = 0;
      break;
    }
  }
  if(bala.x >= LIMITE_DERECHO){
    initBullet();
  }
}

void moverArriba(){
  if(tank.y > 20 && colision(1,tank.x,tank.y,G_JUGADOR_00_W,G_JUGADOR_00_H) == 0){
    tank.y = tank.y - 2;
    tank.mover = SI;
    tank.sprite = g_jugador_09;
    if(bala.disparada == 0){
      bala.x = tank.x;
      bala.y = tank.y;
    }
  }
}

void shootUp(){
  u8 i = 0;
  for(i = 0; i < 3 && getVivoE(i) == 0; i++){
    if(comprobarMuerte(bala.x,bala.y,getXenemy(i),getYenemy(i),3,6) == 1){
      if(colision(1,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H) == 0){
        if(bala.disparada == 0){
          bala.y = tank.y - 14;
          bala.sprite = g_bulletp_0;
          bala.disparada = 1;
        }
        else{
          bala.y = bala.y-4;
          reDrawBullet();
        }
      }
      else{
        modificarMapa(1,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H);
        bala.x = tank.x;
        bala.y = tank.y;
        deleteBullet();
        bala.disparada = 0;
        break;
      }
    }
    else{
      matarE(i);
      bala.x = tank.x;
      bala.y = tank.y;
      deleteBullet();
      bala.disparada = 0;
      break;
    }
  }
  if(bala.y <= 20){
    initBullet();
  }
}

void moverAbajo(){
  if(tank.y < LIMITE_ABAJO && colision(3,tank.x,tank.y,G_JUGADOR_00_W,G_JUGADOR_00_H) == 0){
    tank.y = tank.y + 2;
    tank.mover = SI;
    tank.sprite = g_jugador_03;
    if(bala.disparada == 0){
      bala.x = tank.x;
      bala.y = tank.y;
    }
  }
}

void shootDown(){
  u8 i = 0;
  for(i = 0; i < 3 && getVivoE(i) == 0; i++){
    if(comprobarMuerte(bala.x,bala.y,getXenemy(i),getYenemy(i),3,6) == 1){
      if(colision(3,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H) == 0){
        if(bala.disparada == 0){
          bala.y = tank.y + G_JUGADOR_00_H;
          bala.sprite = g_bulletp_2;
          bala.disparada = 1;
        }
        else{
          bala.y = bala.y + 4;
          reDrawBullet();
        }
      }
      else{
        modificarMapa(3,bala.x,bala.y,G_BULLETP_0_W,G_BULLETP_0_H);
        bala.x = tank.x;
        bala.y = tank.y;
        deleteBullet();
        bala.disparada = 0;
        break;
      }
    }
    else{
      matarE(0);
      bala.x = tank.x;
      bala.y = tank.y;
      deleteBullet();
      bala.disparada = 0;
      break;
    }
  }
  if(bala.y >= LIMITE_ABAJO){
    initBullet();
  }
}
u8 getVivo(){ return tank.vivo; }
u8 getMoverPlayer(){ return tank.mover; }
u8 getVecesPlayer(){ return bala.disparada; }
void setMoverPlayer(u8 n){ tank.mover = n; }
void matarJ(){
  tank.vivo = 1;
  deleteTank();
}
u8 getXplayer(){ return tank.x; }
u8 getYplayer(){ return tank.y; }
void setBala0(){ bala.disparada = 0; }
